// 67d7842dbbe25473c3c32b93c0da8047785f30d78e8a024de1b57352245f9689
/*******************************************************************************
 * Copyright (c) 2007, 2013 Wind River Systems, Inc. and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * Contributors:
 *     Wind River Systems - initial API and implementation
 *******************************************************************************/

/*******************************************************************************
 * Modified by Xilinx,Inc
 * Copyright 2022 Xilinx, Inc. All Rights Reserved. Modifications
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 *******************************************************************************/

package org.eclipse.tcf.internal.dap;

import org.eclipse.tcf.dap.IRequest;
import org.eclipse.tcf.dap.ITCFPresentationProvider;
import org.eclipse.tcf.protocol.Protocol;

import java.util.concurrent.CompletableFuture;
import java.util.logging.Logger;

/**
 * TCFRunnable is a wrapper for IRequest.
 * It implements Runnable interface and is used by TCFModel to handle the request.
 */
public abstract class TCFRunnable implements Runnable {

    protected static final Logger LOGGER = Logger.getLogger(TCFModel.class.getSimpleName());
    private final IRequest request;
    private final Iterable<ITCFPresentationProvider> listeners;
    protected boolean done;

    public TCFRunnable(TCFModel model, IRequest request) {
        this.request = request;
        listeners = model.view_request_listeners;
        if (Protocol.isDispatchThread()) {
            run();
            return;
        }
        if (listeners == null) {
            Protocol.invokeLater(this);
            return;
        }
        Protocol.invokeLater(new Runnable() {
            @Override
            public void run() {
                assert !done;
                TCFRunnable.this.run();
            }
        });
    }

    public void done() {
        assert !done;
        done = true;

        // Don't call Display.asyncExec: display thread can be blocked waiting for the request.
        // For example, display thread is blocked for action state update requests.
        // Calling back into Eclipse on TCF thread is dangerous too - if Eclipse blocks TCF thread
        // we can get deadlocked. Might need a new thread (or Job) to make this call safe.
        request.done();
    }
}
